﻿using System;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Drawing;
using System.Windows;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace ThinkGeoPrintingSample
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            Loaded += MainWindow_Loaded;
        }

        private void MainWindow_Loaded(object sender, RoutedEventArgs e)
        {
            SetupMapWithBlankPage();

            AddMapLayer();

            ExportToBitmap();
        }

        private void SetupMapWithBlankPage()
        {
            // Setup the map unit, you want to always use feet or meters for the printer layout map.
            Map1.MapUnit = GeographyUnit.Meter;

            // Here we create the default zoom levels for the sample.  We have pre-created a PrinterZoomLevelSet
            // That pre-defines commonly used zoom levels based on percentages of zoom
            Map1.ZoomLevelSet = new PrinterZoomLevelSet(Map1.MapUnit,
                                                        PrinterHelper.GetPointsPerGeographyUnit(Map1.MapUnit));

            // Here we set the background color to gray so there is contrast with the white page
            Map1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.StandardColors.LightGray);

            // Create the PrinterInteractiveOverlay to contain all of the PrinterLayers.
            // The interactive overlay allows the layers to be interacted with
            var printerOverlay = new PrinterInteractiveOverlay();

            Map1.InteractiveOverlays.Add("PrintPreviewOverlay", printerOverlay);
            Map1.InteractiveOverlays.MoveToBottom("PrintPreviewOverlay");

            // Create the PagePrinterLayer which shows the page boundary and is the area the user will
            // arrange all of the other layer on top of.
            var pagePrinterLayer = new PagePrinterLayer(PrinterPageSize.AnsiA, PrinterOrientation.Portrait);
            pagePrinterLayer.Open();
            printerOverlay.PrinterLayers.Add("PageLayer", pagePrinterLayer);

            // Get the pages extent, slightly scale it up, and then set that as the default current extent
            Map1.CurrentExtent = AreaBaseShape.ScaleUp(pagePrinterLayer.GetPosition(), 10).GetBoundingBox();

            // Set the minimum sscale of the map to the last zoom level
            Map1.MinimumScale = Map1.ZoomLevelSet.ZoomLevel20.Scale;
        }

        private RectangleShape GetPageBoundingBox(PrintingUnit unit)
        {
            // This helper method gets the pages bounding box in the unit requested
            var printerInteractiveOverlay = (PrinterInteractiveOverlay) Map1.InteractiveOverlays["PrintPreviewOverlay"];
            var pagePrinterLayer = (PagePrinterLayer) printerInteractiveOverlay.PrinterLayers["PageLayer"];
            return pagePrinterLayer.GetPosition(unit);
            ;
        }


        private void AddMapLayer()
        {
            // Create the MapPrinterLayer and set the position
            var mapPrinterLayer = new MapPrinterLayer
                                      {
                                          MapUnit = GeographyUnit.Meter,
                                          BackgroundMask = new AreaStyle(new GeoPen(GeoColor.StandardColors.Black, 1))
                                      };
            //Map Printer Layer is in feet (State Plane Texas North Central Feet NAD83)


            mapPrinterLayer.Open();

            // Set the maps position slightly above the page center and 8 inches wide and 7 inches tall
            RectangleShape pageBoundingbox = GetPageBoundingBox(PrintingUnit.Inch);
            mapPrinterLayer.SetPosition(8, 7, pageBoundingbox.GetCenterPoint().X, pageBoundingbox.GetCenterPoint().Y + 1,
                                        PrintingUnit.Inch);

            //Setup the styles for the parcels layer.                
            var shapeFile = new ShapeFileFeatureLayer(@"..\..\Data\Cardiff_Roads_polyline.shp", ShapeFileReadWriteMode.ReadOnly);
            shapeFile.Open();
            mapPrinterLayer.MapExtent = shapeFile.GetBoundingBox();
            mapPrinterLayer.Layers.Add(shapeFile);

            shapeFile.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.Canal1;
            shapeFile.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            
          
            // Add the MapPrinterLayer to the PrinterInteractiveOverlay
            var printerInteractiveOverlay = (PrinterInteractiveOverlay) Map1.InteractiveOverlays["PrintPreviewOverlay"];
            printerInteractiveOverlay.PrinterLayers.Add("MapLayer", mapPrinterLayer);
            shapeFile.Close();
        }


        private void ExportToBitmap()
        {
            var printerInteractiveOverlay = (PrinterInteractiveOverlay) Map1.InteractiveOverlays["PrintPreviewOverlay"];
            var pagePrinterLayer = printerInteractiveOverlay.PrinterLayers["PageLayer"] as PagePrinterLayer;

            // Create a bitmap that is the size of the pages bounding box.  The page by default is in points unit
            // system which is very similar to pixels so that ampping is nice.
            Bitmap bitmap = null;
            try
            {
                bitmap = new Bitmap((int) pagePrinterLayer.GetBoundingBox().Width,
                                    (int) pagePrinterLayer.GetBoundingBox().Height);

                // Create a GdiPlusGeoCanvas and start the drawing
                var gdiPlusGeoCanvas = new GdiPlusGeoCanvas();
                gdiPlusGeoCanvas.BeginDrawing(bitmap, pagePrinterLayer.GetBoundingBox(), Map1.MapUnit);

                // Loop through all of the PrintingLayer in the PrinterInteractiveOverlay and print all of the
                // except for the PagePrinterLayer                
                var labelsInAllLayers = new Collection<SimpleCandidate>();
                foreach (PrinterLayer printerLayer in printerInteractiveOverlay.PrinterLayers)
                {
                    printerLayer.IsDrawing = true;
                    if (!(printerLayer is PagePrinterLayer))
                    {
                        printerLayer.Draw(gdiPlusGeoCanvas, labelsInAllLayers);
                    }
                    printerLayer.IsDrawing = false;
                }

                // End the drawing
                gdiPlusGeoCanvas.EndDrawing();

                //Save the resulting bitmap to a file and open the file to show the user
                string filename = "PrintingResults.bmp";
                bitmap.Save(filename);
                Process.Start(filename);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message + " " + ex.StackTrace, "Exception");
            }
            finally
            {
                if (bitmap != null)
                {
                    bitmap.Dispose();
                }
            }
        }
    }
}